import aiohttp
import datetime as dt
import pandas as pd
from pprint import pprint

from .DomoAuth import DomoFullAuth
from .DomoDataset import DomoDataset
from .routes import activity_log_routes
from ..utils import convert
from ..utils.DictDot import DictDot


class DomoActivityLog:

    @classmethod
    async def get_activity_log(cls, full_auth: DomoFullAuth,
                               start_time: dt.datetime,
                               end_time: dt.datetime,
                               maximum: int = 1000000,
                               object_type: str = None,
                               session: aiohttp.ClientSession = None,
                               debug: bool = False,
                               convert_to_class_dictdot: bool = False) -> list[DictDot]:

        start_time_epoch = convert.convert_datetime_to_epoch_millisecond(
            start_time)
        end_time_epoch = convert.convert_datetime_to_epoch_millisecond(
            end_time)

        is_close_session = False

        if not session:
            session = aiohttp.ClientSession()
            is_close_session = True

        res_activity_log = await activity_log_routes.search_activity_log(full_auth=full_auth,
                                                                         start_time=start_time_epoch,
                                                                         end_time=end_time_epoch,
                                                                         maximum=maximum,
                                                                         object_type=object_type,
                                                                         session=session,
                                                                         debug=debug
                                                                         )
        if is_close_session:
            await session.close()

        if res_activity_log:
            if convert_to_class_dictdot:
                return [DictDot(al) for al in res_activity_log]
            else:
                df = pd.DataFrame(res_activity_log)
                df["time_dt"] = df["time"].apply(
                    lambda x: convert.convert_epoch_millisecond_to_datetime(x))
                df["instance"] = f'{full_auth.domo_instance}.domo.com'
                df["date"] = df["time_dt"].apply(lambda x: x.date())

            return df

        return None



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoPage.ipynb.

# %% auto 0
__all__ = ['DomoPage', 'PageLayoutTemplate', 'PageLayoutBackground', 'PageLayoutContent', 'PageLayoutStandard',
           'PageLayoutCompact', 'PageLayout', 'DomoPages']

# %% ../../nbs/classes/50_DomoPage.ipynb 2
from fastcore.basics import patch_to
from dataclasses import dataclass, field

import asyncio
import httpx

import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoUser as dmdu
import domolibrary.utils.DictDot as util_dd

import domolibrary.routes.page as page_routes

# %% ../../nbs/classes/50_DomoPage.ipynb 4
@dataclass
class DomoPage:
    id: str
    title: str = None
    parent_page_id: str = None
    top_page_id: str = None
    auth: dmda.DomoAuth = field(default=None, repr=False)
    owners: list = field(default_factory=list)
    cards: list = field(default_factory=list)
    collections: list = field(default_factory=list)
    children: list = field(default_factory=list)
    is_locked: bool = None

    def display_url(self):
        return f"https://{self.auth.domo_instance}.domo.com/page/{self.id}"

    async def _get_domo_users(self, user_id_ls: [str]):
        import domolibrary.classes.DomoUser as dmu

        return await dmu.DomoUsers.by_id(
            user_ids=user_id_ls, only_allow_one=False, auth=self.auth
        )

    async def _get_domo_groups(self, group_id_ls: [str]):
        import domolibrary.classes.DomoGroup as dmg

        return await asyncio.gather(
            *[
                dmg.DomoGroup.get_by_id(group_id=group_id, auth=self.auth)
                for group_id in group_id_ls
            ]
        )

    async def _get_domo_owners_from_dd(self, owners: util_dd.DictDot):
        tasks = list()
        owner_group_ls = [owner.id for owner in owners if owner.type == "GROUP"]

        if owner_group_ls:
            tasks.append(self._get_domo_groups(owner_group_ls))

        owner_user_ls = [owner.id for owner in owners if owner.type == "USER"]

        if owner_user_ls:
            tasks.append(self._get_domo_users(owner_user_ls))

        res = await asyncio.gather(*tasks)

        print(res)

        return res

# %% ../../nbs/classes/50_DomoPage.ipynb 5
@dataclass
class PageLayoutTemplate:
    content_key: int
    x: int
    y: int
    width: int
    height: int
    type: str
    virtual: bool
    virtual_appendix: bool

    @classmethod
    def _from_json(cls, dd):
        return cls(
            content_key=dd.contentKey,
            x=dd.x,
            y=dd.y,
            width=dd.width,
            height=dd.height,
            type=dd.type,
            virtual=dd.virtual,
            virtual_appendix=dd.virtualAppendix,
        )

    def get_body(self):
        return {
            "contentKey": self.content_key,
            "x": self.x,
            "y": self.y,
            "width": self.width,
            "height": self.height,
            "type": self.type,
            "virtual": self.virtual,
            "virtualAppendix": self.virtual_appendix,
        }


@dataclass
class PageLayoutBackground:
    id: int
    crop_height: int
    crop_width: int
    x: int
    y: str
    data_file_id: int
    image_brightness: int
    image_height: int
    image_width: int
    selected_color: str
    text_color: str
    type: str
    is_darkMode: bool
    alpha: float
    src: str

    @classmethod
    def _from_json(cls, dd):
        if dd is not None:
            return cls(
                id=dd.id,
                crop_height=dd.cropHeight,
                crop_width=dd.cropWidth,
                x=dd.x,
                y=dd.y,
                data_file_id=dd.dataFileId,
                image_brightness=dd.imageBrightness,
                image_height=dd.imageHeight,
                image_width=dd.imageWidth,
                selected_color=dd.selectedColor,
                text_color=dd.textColor,
                type=dd.type,
                is_darkMode=dd.darkMode,
                alpha=dd.alpha,
                src=dd.src,
            )
        else:
            return None

    def get_body(self):
        return {
            "id": self.id,
            "cropHeight": self.crop_height,
            "cropWidth": self.crop_width,
            "x": self.x,
            "y": self.y,
            "dataFileId": self.data_file_id,
            "imageBrightness": self.image_brightness,
            "imageHeight": self.image_height,
            "imageWidth": self.image_width,
            "selectedColor": self.selected_color,
            "textColor": self.text_color,
            "type": self.type,
            "darkMode": self.is_darkMode,
            "alpha": self.alpha,
            "src": self.src,
        }


@dataclass
class PageLayoutContent:
    accept_date_filter: bool
    accept_filters: bool
    accept_segments: bool
    card_id: int
    card_urn: str
    compact_interaction_default: bool
    content_key: int
    fit_to_frame: bool
    has_summary: bool
    hide_border: bool
    hide_description: bool
    hide_footer: bool
    hide_margins: bool
    hide_summary: bool
    hide_timeframe: bool
    hide_title: bool
    hide_wrench: bool
    id: int
    summary_number_only: bool
    type: str
    text: str
    background_id: int
    background: PageLayoutBackground

    @classmethod
    def _from_json(cls, dd):
        return cls(
            accept_date_filter=dd.acceptDateFilter,
            accept_filters=dd.acceptFilters,
            accept_segments=dd.acceptSegments,
            card_id=dd.cardId,
            card_urn=dd.cardUrn,
            compact_interaction_default=dd.compactInteractionDefault,
            content_key=dd.contentKey,
            fit_to_frame=dd.fitToFrame,
            has_summary=dd.hasSummary,
            hide_border=dd.hideBorder,
            hide_description=dd.hideDescription,
            hide_footer=dd.hideFooter,
            hide_margins=dd.hideMargins,
            hide_summary=dd.hideSummary,
            hide_timeframe=dd.hideTimeframe,
            hide_title=dd.hideTitle,
            hide_wrench=dd.hideWrench,
            id=dd.id,
            summary_number_only=dd.summaryNumberOnly,
            type=dd.type,
            text=dd.text,
            background_id=dd.backgroundId,
            background=PageLayoutBackground._from_json(dd=dd.background),
        )

    def get_body(self):
        body = {
            "acceptDateFilter": self.accept_date_filter,
            "acceptFilters": self.accept_filters,
            "acceptSegments": self.accept_segments,
            "cardId": self.card_id,
            "cardUrn": self.card_urn,
            "compactInteractionDefault": self.compact_interaction_default,
            "contentKey": self.content_key,
            "fitToFrame": self.fit_to_frame,
            "hasSummary": self.has_summary,
            "hideBorder": self.hide_border,
            "hideDescription": self.hide_description,
            "hideFooter": self.hide_footer,
            "hideMargins": self.hide_margins,
            "hideSummary": self.hide_summary,
            "hideTimeframe": self.hide_timeframe,
            "hideTitle": self.hide_title,
            "hideWrench": self.hide_wrench,
            "id": self.id,
            "summaryNumberOnly": self.summary_number_only,
            "type": self.type,
            "text": self.text,
            "backgroundId": self.background_id,
        }

        if self.background is not None:
            body["background"] = self.background.get_body()
        return body


@dataclass
class PageLayoutStandard:
    aspect_ratio: float
    width: int
    frame_margin: int
    frame_padding: int
    type: str
    template: list[PageLayoutTemplate]

    @classmethod
    def _from_json(cls, dd):
        obj = cls(
            aspect_ratio=dd.aspectRatio,
            width=dd.width,
            frame_margin=dd.frameMargin,
            frame_padding=dd.framePadding,
            type=dd.type,
            template=[],
        )

        if dd.template is not None:
            for template_item in dd.template:
                dc = PageLayoutTemplate._from_json(dd=template_item)
                if dc not in obj.template:
                    obj.template.append(dc)
        return obj


@dataclass
class PageLayoutCompact:
    aspect_ratio: float
    width: int
    frame_margin: int
    frame_padding: int
    type: str
    template: list[PageLayoutTemplate]

    @classmethod
    def _from_json(cls, dd):
        obj = cls(
            aspect_ratio=dd.aspectRatio,
            width=dd.width,
            frame_margin=dd.frameMargin,
            frame_padding=dd.framePadding,
            type=dd.type,
            template=[],
        )
        if dd.template is not None:
            for template_item in dd.template:
                dc = PageLayoutTemplate._from_json(dd=template_item)
                if dc not in obj.template:
                    obj.template.append(dc)
        return obj


@dataclass
class PageLayout:
    id: str
    page_id: str
    is_print_friendly: bool
    is_enabled: bool
    is_dynamic: bool
    has_page_breaks: bool
    content: list[PageLayoutContent]
    standard: PageLayoutStandard
    compact: PageLayoutCompact
    background: PageLayoutBackground

    @classmethod
    def _from_json(cls, dd):
        obj = cls(
            id=dd.layoutId,
            page_id=dd.pageUrn,
            is_print_friendly=dd.printFriendly,
            is_enabled=dd.enabled,
            is_dynamic=dd.isDynamic,
            content=[],
            has_page_breaks=dd.hasPageBreaks,
            standard=PageLayoutStandard._from_json(dd=dd.standard),
            compact=PageLayoutCompact._from_json(dd=dd.compact),
            background=PageLayoutBackground._from_json(dd=dd.background),
        )
        if dd.content is not None:
            for content_item in dd.content:
                dc = PageLayoutContent._from_json(dd=content_item)
                if dc not in obj.content:
                    obj.content.append(dc)
        return obj

    @classmethod
    def generate_new_background_body(cls):
        background_body = {
            "selectedColor": "#EEE000",
            "textColor": "#4A4A4A",
            "type": "COLOR",
            "darkMode": False,
            "alpha": 1,
        }

        return background_body

    def get_body(self):
        body = {
            "layoutId": self.id,
            "pageUrn": self.page_id,
            "printFriendly": self.is_print_friendly,
            "enabled": self.is_enabled,
            "isDynamic": self.is_dynamic,
            "hasPageBreaks": self.has_page_breaks,
            "standard": {
                "aspectRatio": self.standard.aspect_ratio,
                "width": self.standard.width,
                "frameMargin": self.standard.frame_margin,
                "framePadding": self.standard.frame_padding,
                "type": self.standard.type,
            },
            "compact": {
                "aspectRatio": self.compact.aspect_ratio,
                "width": self.compact.width,
                "frameMargin": self.compact.frame_margin,
                "framePadding": self.compact.frame_padding,
                "type": self.compact.type,
            },
        }
        if self.background is not None:
            body["background"] = self.background.get_body()

        if self.content == [] or self.content is None:
            body["content"] = []
        else:
            temp_list = []
            for content_item in self.content:
                temp_list.append(content_item.get_body())
            body["content"] = temp_list

        if self.standard.template is None or self.standard.template == []:
            body["standard"]["template"] = []
        else:
            temp_list = []
            for template_item in self.standard.template:
                temp_list.append(template_item.get_body())
            body["standard"]["template"] = temp_list

        if self.compact.template is None or self.compact.template == []:
            body["compact"]["template"] = []
        else:
            temp_list = []
            for template_item in self.compact.template:
                temp_list.append(template_item.get_body())
            body["compact"]["template"] = temp_list
        return body

# %% ../../nbs/classes/50_DomoPage.ipynb 6
@dataclass
class DomoPage:
    id: str
    title: str = None
    parent_page_id: str = None
    top_page_id: str = None
    auth: dmda.DomoAuth = field(default=None, repr=False)
    owners: list = field(default_factory=list)
    cards: list = field(default_factory=list)
    collections: list = field(default_factory=list)
    children: list = field(default_factory=list)
    is_locked: bool = None
    layout: PageLayout = None

    def display_url(self):
        return f"https://{self.auth.domo_instance}.domo.com/page/{self.id}"

    async def _get_domo_users(self, user_id_ls: [str]):
        import domolibrary.classes.DomoUser as dmu

        return await dmu.DomoUsers.by_id(
            user_ids=user_id_ls, only_allow_one=False, auth=self.auth
        )

    async def _get_domo_groups(self, group_id_ls: [str]):
        import domolibrary.classes.DomoGroup as dmg

        return await asyncio.gather(
            *[
                dmg.DomoGroup.get_by_id(group_id=group_id, auth=self.auth)
                for group_id in group_id_ls
            ]
        )

    async def _get_domo_owners_from_dd(self, owners: util_dd.DictDot):
        tasks = list()
        owner_group_ls = [owner.id for owner in owners if owner.type == "GROUP"]

        if owner_group_ls:
            tasks.append(self._get_domo_groups(owner_group_ls))

        owner_user_ls = [owner.id for owner in owners if owner.type == "USER"]

        if owner_user_ls:
            tasks.append(self._get_domo_users(owner_user_ls))

        res = await asyncio.gather(*tasks)

        return [member for member_ls in res for member in member_ls]

# %% ../../nbs/classes/50_DomoPage.ipynb 7
@patch_to(DomoPage, cls_method=True)
async def _from_bootstrap(cls: DomoPage, page_obj, auth: dmda.DomoAuth = None):
    dd = page_obj
    if isinstance(page_obj, dict):
        dd = util_dd.DictDot(page_obj)

    pg = cls(id=dd.id, title=dd.title, auth=auth)

    if isinstance(dd.owners, list) and len(dd.owners) > 0:
        pg.owners = await pg._get_domo_owners_from_dd(dd.owners)

    if isinstance(dd.children, list) and len(dd.children) > 0:
        pg.children = await asyncio.gather(
            *[
                cls._from_bootstrap(page_obj=child_dd, auth=auth)
                for child_dd in dd.children
                if child_dd.type == "page"
            ]
        )

        [print(other_dd) for other_dd in dd.children if other_dd.type != "page"]

    return pg

# %% ../../nbs/classes/50_DomoPage.ipynb 10
@patch_to(DomoPage, cls_method=True)
async def _from_content_stacks_v3(cls: DomoPage, page_obj, auth: dmda.DomoAuth = None):
    # import domolibrary.classes.DomoCard as dc

    dd = page_obj
    if isinstance(page_obj, dict):
        dd = util_dd.DictDot(page_obj)

    pg = cls(
        id=dd.id,
        title=dd.title,
        parent_page_id=dd.page.parentPageId,
        collections=dd.collections,
        auth=auth,
    )

    if hasattr(dd, "pageLayoutV4") and dd.pageLayoutV4 is not None:
        dd_layout = dd.pageLayoutV4
        pg.layout = PageLayout._from_json(dd=dd.pageLayoutV4)

    if dd.page.owners and len(dd.page.owners) > 0:
        pg.owners = await pg._get_domo_owners_from_dd(dd.page.owners)

    # if dd.cards and len(dd.cards) > 0:
    #     pg.cards = await asyncio.gather(
    #         *[dc.DomoCard.get_from_id(id=card.id, auth=auth) for card in dd.cards])

    return pg


@patch_to(DomoPage, cls_method=True)
async def get_by_id(
    cls: DomoPage,
    page_id: str,
    auth: dmda.DomoAuth,
    return_raw: bool = False,
    debug_api: bool = False,
    include_layout: bool = False,
):
    res = await page_routes.get_page_by_id(
        auth=auth, page_id=page_id, debug_api=debug_api, include_layout=include_layout
    )

    if return_raw:
        return res

    if not res.status == 200:
        return

    pg = await cls._from_content_stacks_v3(page_obj=res.response, auth=auth)

    return pg

# %% ../../nbs/classes/50_DomoPage.ipynb 13
@patch_to(DomoPage, cls_method=True)
async def _from_adminsummary(cls, page_obj, auth: dmda.DomoAuth):
    import domolibrary.classes.DomoCard as dmc

    dd = page_obj
    if isinstance(page_obj, dict):
        dd = util_dd.DictDot(page_obj)

    pg = cls(
        id=dd.id or dd.pageId,
        title=dd.title or dd.pageTitle,
        parent_page_id=dd.parentPageId,
        top_page_id=dd.topPageId,
        collections=dd.collections,
        is_locked=dd.locked,
        auth=auth,
    )

    if dd.page and dd.page.owners and len(dd.page.owners) > 0:
        pg.owners = await pg._get_domo_owners_from_dd(dd.page.owners)

    if dd.cards and len(dd.cards) > 0:
        pg.cards = await asyncio.gather(
            *[dmc.DomoCard.get_from_id(id=card.id, auth=auth) for card in dd.cards]
        )

    return pg

# %% ../../nbs/classes/50_DomoPage.ipynb 14
@patch_to(DomoPage)
async def get_accesslist(
    self,
    auth: dmda.DomoAuth = None,
    is_expand_users: bool = False,
    return_raw: bool = False,
    debug_api: bool = False,
):
    auth = auth or self.auth

    res = await page_routes.get_page_access_list(
        auth=auth, is_expand_users=is_expand_users, page_id=self.id, debug_api=debug_api
    )

    if return_raw:
        return res

    if not res.is_success:
        raise Exception("error getting access list")

    import domolibrary.classes.DomoUser as dmu
    import domolibrary.classes.DomoGroup as dmg

    tasks = await asyncio.gather(
        dmu.DomoUsers.by_id(
            user_ids=[user.get("id") for user in res.response.get("users")],
            only_allow_one=False,
            auth=auth,
        ),
        *[
            dmg.DomoGroup.get_by_id(group_id=group.get("id"), auth=auth)
            for group in res.response.get("groups")
        ]
    )

    res.response.update({"users": tasks[0], "groups": tasks[1:]})

    return res.response

# %% ../../nbs/classes/50_DomoPage.ipynb 17
@patch_to(DomoPage)
async def share(self: DomoPage,
                     auth: dmda.DomoAuth = None,
                     domo_users: list = None,  # DomoUsers to share page with,
                     domo_groups: list = None,  # DomoGroups to share page with
                     message: str = None,  # message for automated email
                     debug_api: bool = False, session: httpx.AsyncClient = None):

    import domolibrary.routes.datacenter as datacenter_routes

    if domo_groups : domo_groups = domo_groups if isinstance(domo_groups, list) else [domo_groups]
    if domo_users : domo_users = domo_users if isinstance(domo_users, list) else [domo_users]


    res = await datacenter_routes.share_resource(
        auth=auth or self.auth,
        resource_ids=[self.id],
        resource_type=datacenter_routes.ShareResource_Enum.PAGE,
        group_ids=[group.id for group in domo_groups] if domo_groups else None,
        user_ids=[user.id for user in domo_users] if domo_users else None,
        message=message,
        debug_api=debug_api, session=session,
    )

    return res


# %% ../../nbs/classes/50_DomoPage.ipynb 20
@dataclass
class DomoPages:
    pass

# %% ../../nbs/classes/50_DomoPage.ipynb 21
@patch_to(DomoPages, cls_method=True)
async def get_pages(
    cls: DomoPages,
    auth=dmda.DomoAuth,
    return_raw: bool = False,
    debug_loop: bool = False,
    debug_api: bool = False,
    session: httpx.AsyncClient = None,
):
    is_close_session = False if session else True

    session = session or httpx.AsyncClient()

    try:
        res = await page_routes.get_pages_adminsummary(
            auth=auth, debug_loop=False, debug_api=False, session=session
        )

        if return_raw:
            return res

        if not res.is_success:
            raise Exception("unable to retrieve pages")

        return await asyncio.gather(
            *[
                DomoPage._from_adminsummary(page_obj, auth=auth)
                for page_obj in res.response
            ]
        )

    finally:
        if is_close_session:
            await session.aclose()

# %% ../../nbs/classes/50_DomoPage.ipynb 24
from datetime import datetime
from utils import convert


@patch_to(DomoPage, cls_method=True)
async def update_layout(
    cls, auth: dmda.DomoAuth, body: dict, layout_id: str, debug_api: bool = False
):
    datetime_now = datetime.now()
    start_time_epoch = convert.convert_datetime_to_epoch_millisecond(datetime_now)

    res_writelock = await page_routes.put_writelock(
        auth=auth,
        layout_id=layout_id,
        user_id=auth.user_id,
        epoch_time=start_time_epoch,
    )
    if res_writelock.status == 200:
        res = await page_routes.update_page_layout(
            auth=auth, body=body, layout_id=layout_id, debug_api=debug_api
        )

        if res.status != 200:
            return False

        res_writelock = await page_routes.delete_writelock(
            auth=auth, layout_id=layout_id
        )
        if res_writelock.status != 200:
            return False

    else:
        return False

    return True



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/classes/50_DomoRole.ipynb.

# %% auto 0
__all__ = ['DomoRole', 'SetRoleGrants_MissingGrants', 'AddUser_Error', 'DeleteRole_Error', 'DomoRoles', 'SearchRole_NotFound',
           'CreateRole_Error']

# %% ../../nbs/classes/50_DomoRole.ipynb 2
from dataclasses import dataclass, field
import httpx
import asyncio


from fastcore.basics import patch_to

import domolibrary.client.DomoAuth as dmda
import domolibrary.classes.DomoInstanceConfig as dic
import domolibrary.client.DomoError as de

import domolibrary.utils.DictDot as util_dd
import domolibrary.routes.role as role_routes

import domolibrary.classes.DomoUser as dmu
import domolibrary.classes.DomoGrant as dmg

# %% ../../nbs/classes/50_DomoRole.ipynb 3
@dataclass
class DomoRole:
    auth: dmda.DomoAuth = field(repr = False)

    id: str
    name: str = None
    description: str = None
    is_system_role: bool = False
    is_default_role: bool = False

    grant_ls: [dmg.DomoGrant] = field(default_factory=list)
    membership_ls: list = field(default_factory=list)

    def __post_init__(self):
        self.is_system_role = True if self.id <= 5 else 0
        
        if self.grant_ls:
            self.grant_ls = self._valid_grant_ls(self.grant_ls)
    
    @staticmethod
    def _valid_grant_ls(grant_ls ) -> [dmg.DomoGrant]:

        if isinstance(grant_ls[0], str):
            return [dmg.DomoGrant(grant_str) for grant_str in grant_ls]
        
        elif isinstance(grant_ls[0], dmg.DomoGrant):
            return grant_ls
    

    # @classmethod
    # def _from_str(cls, id, name, description=None, auth: DomoAuth = None):

    #     return cls(id=id,
    #             name=name,
    #             description=description,
    #             auth=auth
    #             )

    @classmethod
    def _from_json(cls, obj, auth=dmda.DomoAuth, is_default_role = None):

        dd = obj
        if not isinstance(dd, util_dd.DictDot):
            dd = util_dd.DictDot(obj)

        return cls(id=dd.id,
                   name=dd.name,
                   description=dd.description,
                   auth=auth,
                   is_default_role = is_default_role
                   )


# %% ../../nbs/classes/50_DomoRole.ipynb 5
@patch_to(DomoRole)
async def get_grants(self: DomoRole,
                          auth: dmda.DomoAuth = None,
                          role_id: str = None,
                          debug_api: bool = False,
                          session: httpx.AsyncClient = None,
                          return_raw: bool = False
                          ):

    auth = auth or self.auth
    role_id = role_id or self.id
    res = await role_routes.get_role_grants(auth=auth,
                                            role_id=role_id,
                                            debug_api=debug_api,
                                            session=session)

    if return_raw:
        return res
        
    self.grant_ls = [dmg.DomoGrant(obj) for obj in res.response]

    return self.grant_ls


# %% ../../nbs/classes/50_DomoRole.ipynb 8
class SetRoleGrants_MissingGrants(de.DomoError):
    def __init__(self, role_id, missing_grants: [str], domo_instance):
        super().__init__(domo_instance=domo_instance, entity_id=role_id,
                         message=f"failed to add grants: {', '.join(missing_grants)}")


# %% ../../nbs/classes/50_DomoRole.ipynb 9
@patch_to(DomoRole)
async def set_grants(self: DomoRole,
                     grant_ls: list[dmg.DomoGrant],
                     role_id: str = None,
                     auth: dmda.DomoAuth = None,
                     debug_api: bool = False):

    auth = auth or self.auth
    role_id = role_id or self.id

    domo_grants = self._valid_grant_ls(grant_ls)

    # dmic = dic.DomoInstanceConfig(auth = auth)
    # all_grants = await dmic.get_grants()

    # filtered_grant_ls = []
    
    # for domo_grant in all_grants:
    #     if domo_grant.id in grant_ls:
    #         filtered_grant_ls.append(domo_grant.id)

    #         if domo_grant.depends_on_ls:
    #             for parent_grant_id in domo_grant.depends_on_ls:
    #                 match_grant = next(( domo_grant for domo_grant in all_grants if parent_grant_id == domo_grant.id ))
    #                 if match_grant:
    #                     filtered_grant_ls.append(match_grant.id)

    
    
    # set grants
    res = await role_routes.set_role_grants(auth=auth,
                                            role_id=role_id,
                                            role_grant_ls=[domo_grant.id for domo_grant in domo_grants],
                                            debug_api=debug_api)

    # validate grants
    await asyncio.sleep(2)

    all_grants = await self.get_grants(auth=auth,
                                        debug_api=debug_api)

    missing_grants = [
        grant.id for domo_grant in domo_grants if domo_grant not in all_grants]

    if missing_grants:
        raise SetRoleGrants_MissingGrants(
            role_id=role_id, missing_grants=missing_grants, domo_instance=auth.domo_instance)

    return domo_grants


# %% ../../nbs/classes/50_DomoRole.ipynb 13
@patch_to(DomoRole)
async def get_membership(self,
                         role_id=None,
                         auth: dmda.DomoAuth = None,
                         return_raw:bool = False,
                         debug_api: bool = False, session: httpx.AsyncClient = None):

    auth = auth or self.auth
    role_id = role_id or self.id

    res = await role_routes.get_role_membership(auth=auth,
                                                role_id=role_id,
                                                debug_api=debug_api,
                                                session=session)

    if return_raw:
        return res.response

    membership_ls = [dmu.DomoUser._from_search_json(user_obj = obj, auth = auth)for obj in res.response]

    self.membership_ls = membership_ls

    return membership_ls


# %% ../../nbs/classes/50_DomoRole.ipynb 16
class AddUser_Error(de.DomoError):
    def __init__(self, role_id, domo_instance, user_id, user_name = None):
        user_str = f"{user_id} - {user_name}" if user_name else user_id
        super().__init__(domo_instance = domo_instance, message = f"unable to add {user_str} to role {role_id}")

# %% ../../nbs/classes/50_DomoRole.ipynb 17
@patch_to(DomoRole)
async def add_user(self,
                   user: dmu.DomoUser,
                   role_id: str = None,
                   auth: dmda.DomoAuth = None, debug_api: bool = False,
                   session : httpx.AsyncClient = None
                   ):

    role_id = role_id or self.id
    auth = auth or self.auth

    res = await role_routes.role_membership_add_users(auth=auth,
                                                      role_id=role_id,
                                                      user_list=[user.id],
                                                      debug_api=debug_api,
                                                      session = session
                                                      )

    domo_members = await self.get_membership(auth=auth or self.auth,
                                             role_id=role_id or self.id,
                                             debug_api=debug_api)
    self.membership_ls = domo_members

    if user not in domo_members:
        raise AddUser_Error(role_id=role_id, domo_instance=auth.domo_instance,
                            user_id=user.id, user_name=user.display_name)

    return domo_members


# %% ../../nbs/classes/50_DomoRole.ipynb 21
@patch_to(DomoRole)
async def update_role_metadata(self : DomoRole,
                               auth: dmda.DomoAuth = None,
                               role_name=None,
                               role_description: str = None,
                               debug_api: bool = False,
                               session: httpx.AsyncClient = None,
                               return_raw: bool = False
                               ):

    auth = auth or self.auth
    role_name = role_name or self.name

    res = await role_routes.update_role_metadata(
        role_id=self.id,
        role_name=role_name,
        role_description=role_description or self.description,
        auth = auth,
        debug_api=debug_api,
        session=session
    )

    if return_raw:
        return res

    role_ls = await role_routes.get_roles(auth = auth)

    domo_role =  next(
        (DomoRole._from_json(role, auth = auth) for role in role_ls.response if role.get('name') == role_name), None)
    
    return domo_role




# %% ../../nbs/classes/50_DomoRole.ipynb 25
class DeleteRole_Error(de.DomoError):
    def __init__(self,
                 role_id,
                 domo_instance):
        super().__init__(
            message=f'role: {role_id} not found', domo_instance=domo_instance)


# %% ../../nbs/classes/50_DomoRole.ipynb 26
@patch_to(DomoRole, cls_method=True)
async def delete_role(cls: DomoRole,
                      role_id: int,
                      auth: dmda.DomoAuth = None,
                      debug_api: bool = False,
                      session: httpx.AsyncClient = None,
                      ):

    domo_res = await role_routes.get_roles(auth=auth)

    domo_role = next((role for role in domo_res.response if role.get('id') == role_id), None)

    if not domo_role:
        raise DeleteRole_Error(
            role_id=role_id, domo_instance=auth.domo_instance)

    return await role_routes.delete_role(
        role_id= role_id,
        auth=auth,
        debug_api=debug_api,
        session=session
    )


# %% ../../nbs/classes/50_DomoRole.ipynb 30
@dataclass
class DomoRoles:
    auth: dmda.DomoAuth


# %% ../../nbs/classes/50_DomoRole.ipynb 31
@patch_to(DomoRoles, cls_method=True)
async def get_roles(cls: DomoRoles,
                    auth: dmda.DomoAuth,
                    debug_api: bool = False,
                    session: httpx.AsyncClient = None,
                    return_raw: bool = False
                    ):

    res = await role_routes.get_roles(auth=auth, session=session, debug_api=debug_api)

    if return_raw:
        return res

    role_ls = res.response

    default_role_res = await role_routes.get_default_role(auth=auth, session=session, debug_api=debug_api)

    return [DomoRole._from_json(role,
                                auth, 
                                is_default_role=str(default_role_res.response) == str(role.get('id'))) for role in res.response]


# %% ../../nbs/classes/50_DomoRole.ipynb 34
class SearchRole_NotFound(de.DomoError):
    def __init__(self, domo_instance,
                 role_id,
                 message='not found',
                 function_name='search_role'):

        super().__init__(domo_instance=domo_instance, message=message,
                         entity_id=role_id, function_name=function_name)


# %% ../../nbs/classes/50_DomoRole.ipynb 35
@patch_to(DomoRoles, cls_method=True)
async def search_role(cls: DomoRoles,
                    auth: dmda.DomoAuth,
                    role_name: str = None,
                    role_id : str = None,
                    debug_api: bool = False,
                    session: httpx.AsyncClient = None,
                    return_raw: bool = False
                    ):

    all_roles = await DomoRoles.get_roles( auth=auth)

    if role_name:
        domo_role = next(
            (role for role in all_roles if role.name == role_name), None)
    
    if role_id:
        domo_role = next(
            (role for role in all_roles if str(role.id) == str(role_id)), None)

    if not domo_role:
        raise SearchRole_NotFound(domo_instance=auth.domo_instance, role_id=role_name)

    return domo_role

# %% ../../nbs/classes/50_DomoRole.ipynb 39
class CreateRole_Error(de.DomoError):
    def __init__(self, domo_instance,
                 role_id,
                 message,
                 status,
                 function_name='create_role'):

        super().__init__(domo_instance=domo_instance,
                         message=message, status=status,
                         entity_id=role_id,
                         function_name=function_name)


# %% ../../nbs/classes/50_DomoRole.ipynb 40
@patch_to(DomoRoles, cls_method=True)
async def create_role(cls: DomoRoles,
                      auth: dmda.DomoAuth,
                      name: str,
                      description: str = None,
                      debug_api: bool = False,
                      session : httpx.AsyncClient = None
                      ):

    res = await role_routes.create_role(auth=auth,
                                        name=name,
                                        description=description,
                                        debug_api=debug_api,
                                        session = session
                                        )
                                    
    if not res.is_success:
        role_res = await cls.get_roles(auth = auth)
        
        domo_role = next((role for role in role_res if role.name == name))

        if domo_role:
            raise CreateRole_Error(
                domo_instance = auth.domo_instance,
                role_id = name,
                message = "role already exists",
                status = res.status,
        )

        raise CreateRole_Error(
            domo_instance = auth.domo_instance,
            role_id = name,
            message = res.response,
            status = res.status,
        )
    
    return await DomoRoles.search_role(auth = auth, role_name= name)
    

# %% ../../nbs/classes/50_DomoRole.ipynb 43
@patch_to(DomoRoles, cls_method=True)
async def upsert_role(cls: DomoRoles,
                      auth: dmda.DomoAuth,
                      name: str,
                      description: str = None,
                      grant_ls : [dmg.DomoGrant] = None,
                      debug_api: bool = False,
                      debug_prn: bool = False,
                      session: httpx.AsyncClient = None
                      ):

    try:
        domo_role = await DomoRoles.search_role(role_name=name,
                                                auth=auth,
                                                debug_api=debug_api,
                                                session=session,
                                                )

        if domo_role:
            await domo_role.update_role_metadata(
                role_name=name,
                role_description=description,
                session=session
            )
        

    except SearchRole_NotFound as e:
        if debug_prn:
            print(
                f'No role match -- creating new role in {auth.domo_instance}')

        domo_role = await DomoRoles.create_role(auth=auth,
                                           name=name, description=description,
                                           debug_api=debug_api, session=session)
    
    finally:
        if grant_ls:
            grant_ls = domo_role._valid_grant_ls(grant_ls)
            await domo_role.set_grants(grant_ls= grant_ls)
            

        return domo_role


# %% ../../nbs/classes/50_DomoRole.ipynb 47
@patch_to(DomoRole)
async def set_as_default_role(self: DomoRole, debug_api: bool = False, session : httpx.AsyncClient = None):
    return await role_routes.set_default_role(auth=self.auth,
                                                role_id=self.id,
                                                debug_api=debug_api, session = session
                                                )




# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/integrations/DomoJupyter.ipynb.

# %% auto 0
__all__ = ['GetJupyter_ErrorRetrievingAccount', 'GetJupyter_ErrorRetrievingAccountProperty', 'get_jupyter_account',
           'NoConfigCompanyError', 'GetInstanceConfig', 'InvalidAccountTypeError', 'DomoJupyterAccount_InstanceAuth',
           'GetDomains_Query_AuthMatch_Error', 'InvalidAccountNameError', 'GenerateAuth_InvalidDomoInstanceList',
           'GenerateAuth_CredentialsNotProvided', 'GetDomains_Query_Exception_PW_Col_Error']

# %% ../../nbs/integrations/DomoJupyter.ipynb 2
from dataclasses import dataclass, field
from typing import Optional
from enum import Enum


import re
import time
import json

import pandas as pd

from fastcore.basics import patch_to

import domolibrary.client.DomoAuth as dmda
import domolibrary.client.Logger as lc
import domolibrary.classes.DomoDataset as dmds


# %% ../../nbs/integrations/DomoJupyter.ipynb 4
class GetJupyter_ErrorRetrievingAccount(Exception):
    def __init__(self, account_name):
        self.message = f"failure to retrieve DomoDomoJupyter Account {account_name}"
        super().__init__(self.message)


class GetJupyter_ErrorRetrievingAccountProperty(Exception):
    def __init__(self, account_name, property_name):
        self.message = f"failure to retrieve {property_name} DomoDomoJupyter Account {account_name}"
        super().__init__(self.message)


def get_jupyter_account(
    account_name: str,  # name of account as it appears in the
    domojupyter_fn: callable,
    maximum_retry: int = 10,
) -> (
    list,
    dict,
):  # returns account properties list and a dictionary of the properties.
    """import a domojupyter account, will loop until success"""
    account_properties = None

    retry_attempt = 0
    while not account_properties and retry_attempt <= maximum_retry:
        try:
            account_properties = domojupyter_fn.get_account_property_keys(account_name)
            retry_attempt += 1

        except Exception as e:
            print(f"Error:  retry attempt {retry_attempt} - {account_name}: {e}")
            time.sleep(2)

    if not account_properties:
        raise GetJupyter_ErrorRetrievingAccount(account_name=account_name)

    obj = {}

    retry_attempt = 0
    for index, prop in enumerate(account_properties):
        value = None

        while not value and retry_attempt <= maximum_retry:
            try:
                value = domojupyter_fn.get_account_property_value(
                    account_name, account_properties[index]
                )

            except Exception as e:
                print(f"trying again - {prop} - {e}")
                time.sleep(2)

        if not value:
            raise GetJupyter_ErrorRetrievingAccountProperty(
                account_name=account_name, property_name=prop
            )

        obj.update({prop: value})

    return account_properties, obj

# %% ../../nbs/integrations/DomoJupyter.ipynb 5
class NoConfigCompanyError(Exception):
    def __init__(self, sql, domo_instance):
        message = f'SQL "{sql}" returned no results in {domo_instance}'
        self.message = message
        super().__init__(self.message)


class GetInstanceConfig:
    config: pd.DataFrame = None
    logger: lc.Logger = None

    def __init__(self, logger: Optional[lc.Logger] = None):

        self.logger = logger or lc.Logger(app_name="GetInstanceConfig")

    async def _retrieve_company_ds(
        self,
        config_auth: dmda.DomoAuth,
        dataset_id: str,
        sql: str,
        debug_prn: bool = False,
        debug_api: bool = False,
        debug_log: bool = False,
    ) -> pd.DataFrame:  # dataframe of config query
        """wrapper for `DomoDataset.query_dataset_private` retrieves company configuration dataset and stores it as config"""

        ds = await dmds.DomoDataset.get_from_id(
            auth=config_auth, dataset_id=dataset_id, debug_api=debug_api
        )

        message = (
            f"⚙️ START - Retrieving company list \n{ds.display_url()} using \n{sql}"
        )

        if debug_prn:
            print(message)

        self.logger.log_info(message, debug_log=debug_log)

        config_df = await ds.query_dataset_private(
            auth=config_auth, dataset_id=dataset_id, sql=sql, debug_api=debug_api,
            loop_until_end = True
        )
        if len(config_df.index) == 0:
            raise NoConfigCompanyError(sql, domo_instance=config_auth.domo_instance)

        self.config = config_df

        message = f"\n⚙️ SUCCESS 🎉 Retrieved company list \nThere are {len(config_df.index)} companies to update"

        if debug_prn:
            print(message)
        self.logger.log_info(message, debug_log=debug_log)

        return config_df

# %% ../../nbs/integrations/DomoJupyter.ipynb 8
class InvalidAccountTypeError(Exception):
    """raised when account type is not expected type"""

    def __init__(self, account_name, account_type):

        self.message = f"account: {account_name} is not {account_type}"
        super().__init__(self.message)


@dataclass
class DomoJupyterAccount_InstanceAuth:
    """class for interacting with DomoJupyterAccount objects and generating a DomoAuth object"""

    account_name: str

    domo_username: str = None

    display_name: str = field(repr=False, default=None)

    domo_instance: str = field(repr=False, default=None)
    domo_instance_ls: list = field(repr=False, default=None)

    raw_cred: dict = field(repr=False, default=None)
    domo_password: str = field(repr=False, default=None)
    domo_access_token: str = field(repr=False, default=None)

    auth_ls: list = field(repr=False, default=None)

    account_name_mask = "^dj_.*_acc"

    def __post_init__(self):
        if not self.display_name and self.domo_username:
            self._set_display_name()

    def _set_display_name(self):
        clean_text = re.sub("@.*$", "", self.domo_username)
        self.display_name = clean_text

    @staticmethod
    def _test_regex_mask(
        test_string: str,  # the string to test
        regex_mask: str,  # the regex expression to test
    ) -> bool:  # boolean of the re match
        """tests if a string matches the regex pattern"""

        return bool(re.match(regex_mask, test_string))

    @staticmethod
    def _clean_account_admin_accounts(account_name):

        clean_str = re.sub("^dj_", "", account_name)
        clean_str = re.sub("_acc$", "", clean_str)

        return clean_str

# %% ../../nbs/integrations/DomoJupyter.ipynb 10
class GetDomains_Query_AuthMatch_Error(Exception):
    """raise if SQL query fails to return column named 'auth_match_col'"""

    def __init__(self, sql: str = None, domo_instance: str = None, message: str = None):
        self.message = (
            message
            or f"Query failed to return a column 'auth_match_col' sql = {sql} in {domo_instance}"
        )
        super().__init__(self.message)


@patch_to(GetInstanceConfig, cls_method=True)
async def get_domains_with_instance_auth(
    cls: GetInstanceConfig,
    default_auth: dmda.DomoAuth,  # default auth to use with each row
    auth_enum: Enum,  # Enum where enum_name should match to `auth_match_col` from config_sql query and enum_value is the appropriate DomoAuth or DomoJupyterAccount object
    config_auth: dmda.DomoAuth = None,  # which instance to retrieve configuration data from
    config_dataset_id: str = None,  # dataset_id to run config_sql query against
    config_sql: str = "select domain as domo_instance,concat(config_useprod, '-', project) as auth_match_col from table",
    config_df: pd.DataFrame = None,
    debug_api: bool = False,
    debug_log: bool = False,
    debug_prn: bool = False,
    logger: lc.Logger = None,  # pass in Logger class
) -> pd.DataFrame:  # returns a dataframe with domo_instance, instance_auth, and binary column is_valid
    """uses a sql query to retrieve a list of domo_instances and map authentication object to each instance"""

    if not logger:
        logger = lc.Logger(app_name="get_domains_with_instance_auth")

    gic = cls(logger=logger)

    config_df = config_df if isinstance(config_df, pd.DataFrame) else await gic._retrieve_company_ds(
        config_auth=config_auth,
        dataset_id=config_dataset_id,
        sql=config_sql,
        debug_prn=debug_prn,
        debug_log=debug_log,
        debug_api=debug_api,
    )

    if "auth_match_col" not in config_df.columns:
        message = f"Query failed to return a column 'auth_match_col' sql = {config_sql} in {config_auth.domo_instance}"
        raise GetDomains_Query_AuthMatch_Error(message)

    for index, instance in config_df.iterrows():

        match_auth = next(
            (
                member.value
                for member in auth_enum
                if member.name == instance["auth_match_col"]
            )
        )

        creds = match_auth or default_auth

        domo_instance = instance["domo_instance"]

        creds.domo_instance = domo_instance

        if isinstance(creds, DomoJupyterAccount_InstanceAuth):
            creds = creds._generate_auth(domo_instance=domo_instance)

        try:
            await creds.get_auth_token(debug_api=debug_api)
            config_df.at[index, "is_valid"] = 1

        except dmda.InvalidCredentialsError as e:
            if debug_prn:
                print(e)

            logger.log_error(str(e))
            config_df.at[index, "is_valid"] = 0

        config_df.at[index, "instance_auth"] = creds

    return config_df

# %% ../../nbs/integrations/DomoJupyter.ipynb 13
class InvalidAccountNameError(Exception):
    """raised when account name does not follow format string"""

    def __init__(self, account_name=None, regex_pattern=None):
        account_str = f'"{account_name}" '
        regex_str = f'"{regex_pattern}"'

        message = f"string {account_str if account_name else ''}does not match regex pattern {regex_str or ''}"
        self.message = message

        super().__init__(self.message)


@patch_to(DomoJupyterAccount_InstanceAuth, cls_method=True)
def get_domo_instance_auth_account(
    cls: DomoJupyterAccount_InstanceAuth,
    account_name: str,  # domojupyter account to retrieve
    # Domo's domojupyter module, pass in b/c can only be retrieved inside Domo jupyter notebook environment
    domojupyter_fn: callable,
    # set the domo_instance or retrieve from the domojupyter_account credential store
    domo_instance=None,
):
    """
    retrieves Abstract Credential Store from DomoJupyter environment.
    expects credentials property to contain DOMO_USERNAME, DOMO_PASSWORD, or DOMO_ACCESS_TOKEN, and (optional) DOMO_INSTANCE
    """

    if not cls._test_regex_mask(account_name, cls.account_name_mask):
        raise InvalidAccountNameError(
            account_name=account_name, regex_pattern=cls.account_name_mask
        )

    account_properties, dj_account = get_jupyter_account(
        account_name, domojupyter_fn=domojupyter_fn
    )

    if account_properties != ["credentials"]:
        raise InvalidAccountTypeError(
            account_name=account_name, account_type="abstract_credential_store"
        )

    creds = json.loads(dj_account.get("credentials"))

    return cls(
        account_name=account_name,
        raw_cred=creds,
        domo_username=creds.get("DOMO_USERNAME"),
        domo_password=creds.get("DOMO_PASSWORD"),
        domo_access_token=creds.get("DOMO_ACCESS_TOKEN"),
        domo_instance=domo_instance or creds.get("DOMO_INSTANCE"),
    )

# %% ../../nbs/integrations/DomoJupyter.ipynb 15
class GenerateAuth_InvalidDomoInstanceList(Exception):
    def __init__(self):
        message = "provide a list of domo_instances"
        super().__init__(message)


class GenerateAuth_CredentialsNotProvided(Exception):
    def __init__(self):
        message = "object does not have a valid combination of credentials (access_token or username and password)"
        super().__init__(message)


@patch_to(DomoJupyterAccount_InstanceAuth)
def _generate_auth(self, domo_instance):
    if self.domo_access_token:
        auth = dmda.DomoTokenAuth(
            domo_instance=domo_instance, domo_access_token=self.domo_access_token
        )

    elif self.domo_username and self.domo_password:

        auth = dmda.DomoFullAuth(
            domo_instance=domo_instance,
            domo_username=self.domo_username,
            domo_password=self.domo_password,
        )

    else:
        raise GenerateAuth_CredentialsNotProvided()

    return auth


@patch_to(DomoJupyterAccount_InstanceAuth)
def generate_auth_ls(
    self: DomoJupyterAccount_InstanceAuth,
    domo_instance_ls: list[str] = None,  # list of domo_instances
) -> list[dmda.DomoAuth]:  # list of domo auth objects

    """for every domo_instance in domo_instance_ls generates an DomoAuth object"""

    # reset internal lists
    self.domo_instance = None

    self.domo_instance_ls = list(set(domo_instance_ls or self.domo_instance_ls))

    if not self.domo_instance_ls:
        raise GenerateAuth_InvalidDomoInstanceList()

    self.auth_ls = []
    for domo_instance in self.domo_instance_ls:
        auth = self._generate_auth(domo_instance)

        self.auth_ls.append(auth)

    return self.auth_ls

# %% ../../nbs/integrations/DomoJupyter.ipynb 20
class GetDomains_Query_Exception_PW_Col_Error(Exception):
    """raise if SQL query fails to return column named 'config_exception_pw'"""

    def __init__(self, sql: str = None, domo_instance: str = None, message: str = None):
        message = (
            message
            or f"Query failed to return a column 'config_exception_pw' sql = {sql} in {domo_instance}"
        )
        super().__init__(self, message)


@patch_to(GetInstanceConfig, cls_method=True)
async def get_domains_with_global_config_auth(
    cls: GetInstanceConfig,
    config_dataset_id: str,
    config_auth: dmda.DomoAuth,  # which instance to retrieve configuration data from
    global_auth: dmda.DomoAuth,  # global authentication credentials
    global_exception_auth: dmda.DomoAuth,  # exception credentials (ex 24 char password)
    # must return a column named domo_instance, if there is an exception_auth, must return a column 'config_exception_pw'
    config_sql: str = "select domain as domo_instance, config_exception_pw from table",
    debug_api: bool = False,
    debug_log: bool = False,
    debug_prn: bool = False,
    logger: lc.Logger = None,
) -> pd.DataFrame:
    if not logger:
        logger = lc.Logger(app_name="get_domains_with_global_config_auth")

    gic = cls(logger=logger)

    df = await gic._retrieve_company_ds(
        config_auth=config_auth,
        dataset_id=config_dataset_id,
        sql=config_sql,
        debug_prn=debug_prn,
        debug_log=debug_log,
        debug_api=debug_api,
    )

    if "config_exception_pw" not in df.columns:
        message = f"Query failed to return a column 'config_exception_pw' sql = {config_sql} in {config_auth.domo_instance}"
        gic.logger.log_error(message)
        raise GetDomains_Query_Exception_PW_Col_Error(message=message)

    for index, instance in df.iterrows():
        creds = global_auth

        if instance["config_exception_pw"] == 1:
            creds = global_exception_auth

        creds.domo_instance = instance["domo_instance"]

        try:
            await creds.get_auth_token()
            df.at[index, "is_valid"] = 1

        except dmda.InvalidCredentialsError as e:
            if debug_prn:
                print(e)

            logger.log_error(str(e))
            df.at[index, "is_valid"] = 0

        finally:
            df.at[index, "instance_auth"] = creds

    return df



import aiohttp

import Library.DomoClasses.DomoAuth as dmda
import Library.utils.ResponseGetData as rgd

from Library.DomoClasses.routes.get_data import get_data


async def get_shared_repos(full_auth: dmda.DomoFullAuth, session: aiohttp.ClientSession = None, debug: bool = False) -> rgd.ResponseGetData:
    url = f"https://{full_auth.domo_instance}.domo.com/api/version/v1/repositories/search"
    body = {
        "query": {
            "offset": 0,
            "limit": 50,
            "fieldSearchMap": {},
            "sort": "lastCommit",
            "order": "desc",
            "filters": {
                "userId": None
            },
            "dateFilters": {}
        },
        "shared": False
    }

    res = await get_data(auth=full_auth,
                         method='POST',
                         url=url,
                         body=body,
                         session=session,
                         debug=debug)

    return res


async def get_repo_from_id(full_auth: dmda.DomoFullAuth,
                           repository_id: str,
                           debug: bool = False) -> rgd.ResponseGetData:

    url = f"https://{full_auth.domo_instance}.domo.com/api/version/v1/repositories/{repository_id}"

    return await get_data(auth=full_auth,
                          method='GET',
                          url=url,
                          debug=debug)


async def get_shared_repos(full_auth: dmda.DomoFullAuth, session: aiohttp.ClientSession = None, debug: bool = False) -> rgd.ResponseGetData:
    url = f"https://{full_auth.domo_instance}.domo.com/api/version/v1/repositories/search"
    body = {
        "query": {
            "offset": 0,
            "limit": 50,
            "fieldSearchMap": {},
            "sort": "lastCommit",
            "order": "desc",
            "filters": {
                "userId": None
            },
            "dateFilters": {}
        },
        "shared": False
    }

    res = await get_data(auth=full_auth,
                         method='POST',
                         url=url,
                         body=body,
                         session=session,
                         debug=debug)

    return res
